/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - main.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-fileio.h"
#include "sussen-gconf.h"
#include "sussen-util.h"
#include "sussen-main-callback.h"
#include "sussen-getting-started-druid.h"

/* Nessus includes */

#include <includes.h>
#include "globals.h"
#include "nessus.h"
#include "filter.h"

/* ADAM */
#include "preferences.h"
#include "backend.h"

/* Nessus global variables */

struct arglist *Plugins = NULL;
struct arglist *Scanners = NULL;
struct arglist *Dependencies = NULL;
struct arglist *Upload = NULL;

harglst * Sessions = NULL;
int Sessions_saved = 0;
int Detached_sessions_saved = 0;
int DetachedMode = 0;
int PluginsNum;
int ScannersNum;
struct arglist * Prefs;
struct arglist * MainDialog;
struct arglist * ArgSock;
char * Alt_rcfile = NULL;
struct plugin_filter Filter;
int GlobalSocket;
char * stored_pwd = NULL;
int F_show_pixmaps;
int F_quiet_mode;
int F_nessusd_running;
int First_time = 0;
int ListOnly = 0;

/* Sussen globals 
 *
 * FIXME: put all this in a structure or something
 */

GtkWidget *main_window;
GConfClient *client;

GladeXML *mainwindow_xml;
GladeXML *preferences_xml;
GladeXML *kb_dialog;
GladeXML *policy_manager;
GladeXML *report_xml;

GtkWidget *nessusd_tree;
GtkListStore *nessusd_store;

GtkListStore *plug_family_store;
GtkWidget *plug_family_tree;

GtkWidget *plug_tree;
GtkListStore *plug_store;

GtkWidget *targets_tree;
GtkListStore *targets_store;

GtkWidget *session_tree;
GtkListStore *session_store;

GtkWidget *scan_status_tree;
GtkListStore *scan_status_store;

GtkTreeModel *global_model;
GtkTreeIter *global_iter;

GtkWidget *plug_pref_tree;
GtkListStore *plug_pref_store;
GtkWidget *plugin_description_textview;

gint global_tag;
gint global_backend;
gint global_flag;

gint use_policy_file = 0;
GHashTable *plugin_hash;

void load_gnome_frontend (void) {
	
    /* load the interface */
    mainwindow_xml = glade_xml_new(PACKAGE_DATA_DIR"/sussen.glade", NULL, NULL);
	main_window = glade_xml_get_widget (mainwindow_xml,"sussen_main_window");

    /* connect the signals in the interface */
    glade_xml_signal_autoconnect(mainwindow_xml); 

	/* Setup and re-draw tree widget */
	nessusd_tree = glade_xml_get_widget (mainwindow_xml,"server_treeview");
	nessusd_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW(nessusd_tree), GTK_TREE_MODEL (nessusd_store));
	{
		GtkCellRenderer *text_renderer, *pixbuf_renderer;
		GtkTreeViewColumn *column;

		pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();
		column = gtk_tree_view_column_new_with_attributes ("Connected",pixbuf_renderer,
															"pixbuf",0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (nessusd_tree), column);
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Nessus Servers",text_renderer,"text", 1,NULL);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (nessusd_tree), column);		
	}

	update_server_listview ();
	
	/*
	 * Configure scan_status_listview
	 */
	 
	GtkWidget *selected; 
	 
	scan_status_tree = glade_xml_get_widget (mainwindow_xml,"scan_status_listview");
	selected = GTK_WIDGET(gtk_tree_view_get_selection (GTK_TREE_VIEW(scan_status_tree)));
	gtk_tree_selection_set_mode (GTK_TREE_SELECTION(selected), GTK_SELECTION_MULTIPLE);
	scan_status_store = gtk_list_store_new (3,G_TYPE_STRING,G_TYPE_STRING,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (scan_status_tree), GTK_TREE_MODEL (scan_status_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer;
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Hostname",text_renderer,
															"text",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);
		gtk_tree_view_append_column (GTK_TREE_VIEW (scan_status_tree), column);	
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Port Scan",text_renderer,
															"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (scan_status_tree), column);
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Attack",text_renderer,
															"text",2,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 2);
		gtk_tree_view_append_column (GTK_TREE_VIEW (scan_status_tree), column);		
	}	
	
	set_widgets_sensitive (FALSE,FALSE);

	GtkWidget *w;

	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_radio");
	g_object_set_data(G_OBJECT(w), "scan_policy_switch", (void*)1);
	
	w = glade_xml_get_widget (mainwindow_xml, "scan_policy_use_current_radio");
	g_object_set_data(G_OBJECT(w), "scan_policy_switch", (void*)2);

	
}

int main(int argc, char * argv[]) {
	
	PluginsNum = 0;
	ScannersNum = 0;
	Scanners = Plugins = MainDialog = NULL;
	ArgSock = NULL;
	GlobalSocket = -1;
	F_quiet_mode = 0;
	F_show_pixmaps = 1;

	bindtextdomain (GETTEXT_PACKAGE, PACKAGE_LOCALE_DIR);
	bind_textdomain_codeset (GETTEXT_PACKAGE, "UTF-8");
	textdomain (PACKAGE);
	
	/* libxml */
	xmlKeepBlanksDefault(0);
		
	/* Get GConfClient */
	client = gconf_client_get_default (); 	

	/* Tell GConfClient we are interested in /apps/nessus */
	gconf_client_add_dir (client,"/apps/sussen",GCONF_CLIENT_PRELOAD_NONE,NULL);
		
	gnome_program_init (PACKAGE,VERSION,LIBGNOMEUI_MODULE,argc,argv,
						 GNOME_PROGRAM_STANDARD_PROPERTIES, NULL);
	F_nessusd_running = 0;

	if (preferences_init(&Prefs)) {
		exit (2);
	} else {
		Prefs = emalloc(sizeof(struct arglist));
	}	

	plugin_hash = g_hash_table_new (g_str_hash,g_str_equal);

	if (!check_for_server_xml()) {
		show_getting_started_druid ();
	} 
		
	/*
  	 * Set up the main window
 	 */

	load_gnome_frontend ();
	gtk_main ();

	close_stream_connection(GlobalSocket); 
	backend_clear_all();
	GlobalSocket = -1;

	/* libxml */
	xmlCleanupParser();
	
	/* This ensures we cleanly detach from the GConf server (assuming
	 * we hold the last reference). It's purely a bit of cleanliness,
	 * the server does survive fine if we crash.
	 */
	g_object_unref (G_OBJECT (client));
	
	return(0);
}
