/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - gconf.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-gconf.h"

void config_entry_commit (GtkWidget *entry) {

	gchar *text;
	const gchar *key;
	GConfClient *client;
  
	client = g_object_get_data (G_OBJECT (entry), "client");  
	text = gtk_editable_get_chars (GTK_EDITABLE (entry), 0, -1);
	key = g_object_get_data (G_OBJECT (entry), "key");

	/* Unset if the string is zero-length, otherwise set */
	if (*text != '\0')
		gconf_client_set_string (client, key, text, NULL);
	else
		gconf_client_unset (client, key, NULL);
  
	g_free (text);	
}

void config_entry (GtkWidget *entry, GConfClient *client, const gchar *config_key, gboolean focus) {

	gchar *str; 

	str = gconf_client_get_string (client,config_key,NULL);
	if (str) {
		gtk_entry_set_text (GTK_ENTRY (entry), str);
		g_free (str);
	}			
	
	g_object_set_data (G_OBJECT (entry), "client", client);
	g_object_set_data_full (G_OBJECT (entry), "key",
                          g_strdup (config_key),
                          (GDestroyNotify) g_free);
		
	/* Commit changes if the user focuses out, or hits enter; we don't
	 * do this on "changed" since it'd probably be a bit too slow to
	 * round-trip to the server on every "changed" signal.
	 */

	g_signal_connect (G_OBJECT (entry), "focus_out_event", 
						G_CALLBACK (config_entry_commit),NULL);

	g_signal_connect (G_OBJECT (entry), "activate",
						G_CALLBACK (config_entry_commit),NULL);  

	/* Set the entry insensitive if the key it edits isn't writable.
	 * Technically, we should update this sensitivity if the key gets
	 * a change notify, but that's probably overkill.
	 */

	gtk_widget_set_sensitive (entry,
			gconf_client_key_is_writable (client,config_key, NULL));

	if (focus)
		gtk_widget_grab_focus (entry);
}
