/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-main-callback.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-auth.h"
#include "sussen-fileio.h"
#include "sussen-gconf.h"
#include "sussen-main-callback.h"
#include "sussen-policy-callback.h"
#include "sussen-preferences.h"
#include "sussen-report.h"
#include "sussen-report-druid.h"
#include "sussen-util.h"


#include <includes.h>
#include "nessus.h"
#include "globals.h"
#include "parser.h"
#include "families.h"
#include "comm.h"
#include "backend.h"
#include "attack.h"
#include "auth.h"
#include "read_target_file.h"

static harglst *plugins_order_table = NULL;

GtkWidget *file_dialog;
GladeXML *xml;
GladeXML *session_xml;
GHashTable *scan_hosts;
gboolean scan_running;
gboolean restore_session;

gchar *nessusd_host, *nessusd_port, *nessusd_user;

static harglst *hosts = NULL;

static void status_input_callback(struct arglist *ctrls, int fd, int condition);
static void status_list_update(struct arglist *ctrls, char *msg, int short_status);
static void build_plugins_order_table(char *order);

static int idle_socket(struct arglist * ctrls) {

	fd_set rd;
	struct timeval tv = {0,100};
	int n, soc;

	if(GlobalSocket < 0) {
		fprintf(stderr, "idle_socket: GlobalSocket=%d\n", GlobalSocket);
		return FALSE;
	}

	soc = nessus_get_socket_from_connection(GlobalSocket);
	if((soc < 0) || (soc >= FD_SETSIZE)) {
		fprintf(stderr, "idle_server: soc(%d) out of range\n", soc);
		return -1;
	}

	FD_ZERO(&rd);
	FD_SET(soc, &rd);
	n = select(soc+1, &rd, NULL, NULL, &tv);
	if(FD_ISSET(soc, &rd)&& (n > 0))
		status_input_callback(ctrls, soc /* ? Not used */, 0);

	return TRUE;
}

static int is_server_present(int soc) {

	fd_set  rd;
	struct timeval tv = {2,0};
	int fd = nessus_get_socket_from_connection(soc);

	if(fd < 0 || fd >= FD_SETSIZE) {
		fprintf(stderr, "is_server_present: fd(%d) out of range\n", fd);
		return 0;
	}
	
	FD_ZERO(&rd);
	FD_SET(fd, &rd);
	
	if(select(fd+1, &rd, NULL, NULL, &tv) > 0) {
		int len = -1;
		ioctl(fd, FIONREAD, &len);
		
		if(!len){
			return 0;
		}
	}

	return 1;
}


gboolean update_each_attack (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GHashTable *hosts) {
	
	gchar *cur_host;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(scan_status_store),iter,0,&cur_host,-1);
		if (strcmp(cur_host,g_hash_table_lookup (hosts,"hostname")) == 0) {
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), iter,
								2, g_strdup(g_hash_table_lookup (hosts,"percent")),
								-1);
		} 
		
		g_free (cur_host);
		return FALSE;
	}
	
	return TRUE;
}


gboolean update_each_portscan (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GHashTable *hosts) {

	gchar *cur_host;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(scan_status_store),iter,0,&cur_host,-1);
		if (strcmp(cur_host,g_hash_table_lookup (hosts,"hostname")) == 0) {
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), iter,
									1, g_strdup(g_hash_table_lookup (hosts,"percent")),
									-1);
		} 
		
		g_free (cur_host);
		return FALSE;
	}
	
	return TRUE;
}

void status_list_update(struct arglist *ctrls, char *msg, int short_status) {
	
	char *hostname;
	char *action;
	char *current = NULL;
	int max;
	gfloat gmax;
	gfloat gcurrent;
	int v = 1;
		
	if(!hosts) {
		hosts = harg_create(65000);
	}

	if(!short_status)
		parse_nessusd_status(msg, &hostname, &action, &current, &max);
	else
		parse_nessusd_short_status(msg, &hostname, &action, &current, &max);
	
	if(hostname) {
		v = harg_get_int(hosts, hostname);
		if(!v){
			harg_add_int(hosts, hostname, 1);
			v = 1;

			GtkTreeIter iter;
			
			gtk_list_store_append (scan_status_store,&iter);
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), &iter,
								0, g_strdup(hostname),
								-1);
			
		}
		else {
			v++;
			harg_set_int(hosts, hostname, v);

			if (!restore_session) {
				gfloat f;

				gmax = max;
				gcurrent = atoi(current);
				f = (gcurrent/gmax);
				if(f>1)
					f=1;
				if(f<0.0)
					f=0.0;		
		
				gint test;
				test = f * 100;
				gchar *test2 = g_malloc (10);
		
				sprintf (test2, "%d%%", test);
				g_hash_table_insert (scan_hosts,"hostname",hostname);
				g_hash_table_insert (scan_hosts,"percent", test2);
		
				if( strcmp(action, "portscan") == 0 ) {
					gtk_tree_model_foreach (GTK_TREE_MODEL(scan_status_store), (void *)update_each_portscan,
											scan_hosts);
				} else {
					gtk_tree_model_foreach (GTK_TREE_MODEL(scan_status_store), (void *)update_each_attack,
											scan_hosts);
				}
				g_free (test2);	
			}
		}
	}
	
	efree(&hostname);
	efree(&action);
	if(current)
		efree(&current);	
}

void build_plugins_order_table(char *order) {

	int num = 0;
	char * t;
	int i = 0;
	int * plugins_order_table_int;
	t = order;

	if(plugins_order_table)
		harg_close(plugins_order_table);


	while((t = strchr(t+1, ';')))num++;
		plugins_order_table_int = emalloc((num+1)*sizeof(int));

	t = order;
	if(num)
		while(t) {
			char * next = strchr(t, ';');

			if(next)next[0]=0;
				plugins_order_table_int[i++] = atoi(t);

			if(next)
				next[0]=';';

			t = next+1;
			next = strchr(next+1, ';');

			if(!next)t = NULL;
		}
 
	/*
	 * Now, create the names table
	 */
	plugins_order_table = harg_create(num*3 + 1);
  
	for(i=0;i<num;i++) {
		struct arglist * plugs = Plugins;
   
		while(plugs && plugs->next && ((int)(arg_get_value(plugs->value, "ID"))!=plugins_order_table_int[i]))
			plugs = plugs->next;

			if(!(plugs && plugs->next)) {
				plugs = Scanners;
				while(plugs && plugs->next && ((int)(arg_get_value(plugs->value, "ID"))!=plugins_order_table_int[i]))
					plugs = plugs->next;
			}

			if(plugs) {
				char * id = emalloc(20);
					
				sprintf(id, "%d", i+1);
				harg_add_string(plugins_order_table, id, plugs->name);
				efree(&id);
			}
	}
	efree(&plugins_order_table_int);	
}

void status_input_callback (struct arglist *ctrls, int fd, int condition) {

	int finished = 0;
	char buf [32678];
	char * msg;
	int n, type = -1;
	int interrupted = 0;
	
	network_gets_raw(buf, 32678);
	if ((n = strlen (buf)) && buf [n-1] == '\n') buf [n-1] = '\0';
	
	if( buf[0] == '\0') {
		if(!is_server_present(GlobalSocket)) {
			interrupted++;
		}
		else 
			return;
	}	

	if (!interrupted) {
		
		msg  = emalloc(strlen(buf)+1);
		type = parse_server_message(buf, (int)arg_get_value(ctrls, "MONITOR_BACKEND"), msg);
 
		switch(type) {
			case MSG_BYE : 
				network_printf("CLIENT <|> BYE <|> ACK\n");
				finished = 1;
					break;
		
			case MSG_STAT2 : 
				status_list_update(ctrls,buf+2, 1);	
				break;

			case MSG_STAT :
				status_list_update(ctrls, msg, 0);
				break;

			case MSG_PLUGINS_ORDER :
				build_plugins_order_table(msg);
				break;

			case MSG_FINISHED :
				break;	
		}

		bzero(buf, sizeof(buf));
		efree(&msg);
		
		if(finished) {
			F_nessusd_running = 0;
			scan_running = FALSE;
			if (restore_session)
				restore_session = FALSE;
		
			gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
			set_widgets_sensitive (TRUE,FALSE);
			GtkWidget *w = glade_xml_get_widget (mainwindow_xml, "stop_scan_button");
			gtk_widget_set_sensitive (w,FALSE);
			w = glade_xml_get_widget (mainwindow_xml, "stop_scan1");
			gtk_widget_set_sensitive (w,FALSE);
		
			gint user_response;
			user_response = show_report_druid ();
		
			switch (user_response) {
			
				case REPORT_DRUID_VIEW:
				case REPORT_DRUID_VIEW_EXPORT:
					show_results ((int)arg_get_value(ctrls, "MONITOR_BACKEND"), interrupted);
					break;
			
				case REPORT_DRUID_EXPORT:
					break;
			
				case REPORT_DRUID_CANCEL:
					break;
				
				default:
					break;
			}
		}
	} else {
			F_nessusd_running = 0;
			scan_running = FALSE;
		
			gtk_idle_remove((int)arg_get_value(ctrls, "TAG"));
			set_widgets_sensitive (TRUE,FALSE);
			GtkWidget *w = glade_xml_get_widget (mainwindow_xml, "stop_scan_button");
			gtk_widget_set_sensitive (w,FALSE);
			w = glade_xml_get_widget (mainwindow_xml, "stop_scan1");
			gtk_widget_set_sensitive (w,FALSE);
					
			update_status ("connection to nessus server lost\n");
	}
}

void store_filename (GtkWidget *widget, gpointer user_data) {

	GtkWidget *entry;
	gchar *filename;
	gchar *uri_filename;
	
	filename = (gchar *)gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	uri_filename = g_malloc (strlen(filename)+strlen("file:")+1);
	sprintf (uri_filename, "file:%s", filename);
	
	entry = glade_xml_get_widget (mainwindow_xml,"targets_entry");
	gtk_entry_set_text (GTK_ENTRY(entry),uri_filename);
	
	g_free (uri_filename);
}

void on_target_browse_button_clicked (GtkWidget *widget, gpointer user_data) {
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Load Targets From File");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (store_filename),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void set_policy_entry (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;
	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	w = glade_xml_get_widget (mainwindow_xml,"policy_entry");
	gtk_entry_set_text (GTK_ENTRY(w),filename);
}

void on_policy_browse_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Select Policy File");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (set_policy_entry),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void on_policy_manager_button_clicked (GtkWidget *widget, gpointer user_data) {

	show_policy_manager ();
}

void on_continuous_scan_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {

	gboolean status;
	GtkWidget *w;
	
	w = glade_xml_get_widget ( xml, "delay_between_scans_entry" );
	
	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	gtk_widget_set_sensitive ( w, status );
}


void on_detached_scan_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {

	gboolean status;
	GtkWidget *w1,*w2;
	
	w1 = glade_xml_get_widget ( xml, "continuous_scan_checkbutton" );
	w2 = glade_xml_get_widget ( xml, "email_results_to_entry" );
	
	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	gtk_widget_set_sensitive ( w1, status );
	gtk_widget_set_sensitive ( w2, status );
}


void on_reuse_kb_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {

	gboolean status;
	GtkWidget *c3,*c4,*c5,*c6,*c7;
	
	c3 = glade_xml_get_widget ( xml, "no_scanners_checkbutton");
	c4 = glade_xml_get_widget ( xml, "no_info_plugins_checkbutton");
	c5 = glade_xml_get_widget ( xml, "no_attack_plugins_checkbutton");
	c6 = glade_xml_get_widget ( xml, "no_dos_plugins_checkbutton");
	c7 = glade_xml_get_widget ( xml, "show_differences_checkbutton"); 

	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));

	gtk_widget_set_sensitive ( c3, status );
	gtk_widget_set_sensitive ( c4, status );
	gtk_widget_set_sensitive ( c5, status );
	gtk_widget_set_sensitive ( c6, status );
	gtk_widget_set_sensitive ( c7, status );	
}

void on_enable_kb_checkbutton_toggled (GtkWidget *widget, gpointer user_data) {
	
	gboolean status;
	GtkWidget *r1,*r2,*r3,*c2,*c3,*c4,*c5,*c6,*c7,*e1;

	r1 = glade_xml_get_widget ( xml, "kb_test_radio");
	r2 = glade_xml_get_widget ( xml, "kb_test_past_hosts_radio");
	r3 = glade_xml_get_widget ( xml, "kb_test_new_hosts_radio");
	c2 = glade_xml_get_widget ( xml, "reuse_kb_checkbutton");
	c3 = glade_xml_get_widget ( xml, "no_scanners_checkbutton");
	c4 = glade_xml_get_widget ( xml, "no_info_plugins_checkbutton");
	c5 = glade_xml_get_widget ( xml, "no_attack_plugins_checkbutton");
	c6 = glade_xml_get_widget ( xml, "no_dos_plugins_checkbutton");
	c7 = glade_xml_get_widget ( xml, "show_differences_checkbutton"); 
	e1 = glade_xml_get_widget ( xml, "max_aged_kb_entry");
	
	status = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(widget));
	if (status) {
		gtk_widget_set_sensitive ( r1, status );
		gtk_widget_set_sensitive ( r2, status );
		gtk_widget_set_sensitive ( r3, status );
		gtk_widget_set_sensitive ( c2, status );
		gtk_widget_set_sensitive ( e1, status );
	} else {
		gtk_widget_set_sensitive ( r1, status );
		gtk_widget_set_sensitive ( r2, status );
		gtk_widget_set_sensitive ( r3, status );
		gtk_widget_set_sensitive ( c2, status );
		gtk_widget_set_sensitive ( c3, status );
		gtk_widget_set_sensitive ( c4, status );
		gtk_widget_set_sensitive ( c5, status );
		gtk_widget_set_sensitive ( c6, status );
		gtk_widget_set_sensitive ( c7, status );
		gtk_widget_set_sensitive ( e1, status );
	}	
}

void arg_addset_value(struct arglist *arglist, char *name, char *value) {

	if(arg_get_type(arglist, name) < 0)
		arg_add_value(arglist, name, ARG_STRING, strlen(value), strdup(value));
	else {
		arg_set_type(arglist, name, ARG_STRING);
		arg_set_value(arglist, name, strlen(value), strdup(value));
	}
}

void on_start_scan_button_clicked (GtkWidget *widget, gpointer user_data) {

	struct arglist *serv;
	gchar expansion[30];
	gchar *s;
	gchar *hostname;
	GtkWidget *w;
		
	bzero (expansion, 30);
	serv = arg_get_value(Prefs, "SERVER_PREFS");
	if (!serv) {
		g_print ("!serv\n");
		serv = emalloc(sizeof(struct arglist));
		arg_add_value(Prefs, "SERVER_PREFS", ARG_ARGLIST, -1, serv);				
	}

	s = estrdup("no");
	if (arg_get_value(serv, "auto_enable_dependencies")) {
		arg_set_value(serv, "auto_enable_dependencies", strlen(s), s);
	} else {
		arg_add_value(serv, "auto_enable_dependencies", ARG_STRING, strlen(s), s);	
	}
		
	w = glade_xml_get_widget (mainwindow_xml,"dns_transfer_checkbutton");
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) )
		strcat (expansion, "dns;");
   
	strcat(expansion, "ip;");    
   
	if (!strlen(expansion))
		strncpy(expansion, "none;", 5);

	expansion[strlen(expansion)-1]=0;	

	w = glade_xml_get_widget (mainwindow_xml,"save_session_checkbutton");
	if ( gtk_toggle_button_get_active GTK_TOGGLE_BUTTON (w) )
		s = strdup("yes");
	else
		s = strdup("no");

	if (arg_get_value(serv, "save_session"))
		arg_set_value(serv, "save_session", strlen(s), s);
	else
		arg_add_value(serv, "save_session", ARG_STRING, strlen(s), s);
	
	w = glade_xml_get_widget (mainwindow_xml,"targets_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	if (s && strlen(s) > 0) {
		hostname = target_translate(s); 
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
   					                      GTK_DIALOG_DESTROY_WITH_PARENT,
                    					  GTK_MESSAGE_ERROR,
										GTK_BUTTONS_CLOSE,
										_("You must specify a host(s) or network(s) to scan"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		return;		
	}
	
	s = emalloc(strlen(expansion)+1);
	strncpy(s, expansion, strlen(expansion));
   
	if(arg_get_value(serv, "host_expansion"))
		arg_set_value(serv, "host_expansion", strlen(s), s);
	else
		arg_add_value(serv, "host_expansion", ARG_STRING, strlen(s), s);	

	GSList *list;
	GtkWidget *button;
	const gchar *filename;
	gint user_response;
		
	button = glade_xml_get_widget (mainwindow_xml, "scan_policy_radio");
	list = gtk_radio_button_get_group (GTK_RADIO_BUTTON(button));

	if (list) {
		while (list) {
			
			GtkWidget *button = list->data;	
			
			if(GTK_TOGGLE_BUTTON(button)->active) {
				
				user_response = (gint)g_object_get_data(G_OBJECT(button), "scan_policy_switch");
				
				switch (user_response) {
				
					case 1:
						w = glade_xml_get_widget (mainwindow_xml,"policy_entry");
						filename = gtk_entry_get_text(GTK_ENTRY(w));
						if (filename && strlen(filename) > 0) {	
							if (!load_policy_xml (filename))
								return;
							
						} else {
							GtkWidget *dialog;

							dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		    					                      GTK_DIALOG_DESTROY_WITH_PARENT,
            		                    					  GTK_MESSAGE_ERROR,
																GTK_BUTTONS_CLOSE,
																_("You must specify a policy filename"));
					 		gtk_dialog_run (GTK_DIALOG (dialog));
							gtk_widget_destroy (dialog);		
							return;
						}
						break;

					case 2:
						break;
					
					default:
						break;
				}
			}
			list = list->next;
		}
	}	
	
	/* error check */
	if(GlobalSocket < 0) {
		update_status ("You must connect to a nessusd host before you start");
	}
	
 	if(!strlen(hostname)) { 
		update_status ("You must enter the name of the primary target");
	}
 		
	struct arglist *ctrls = emalloc(sizeof(struct arglist));
	int tag;
	int backend = backend_init(NULL);
		
	global_backend = backend;
	arg_add_value(ctrls, "MONITOR_BACKEND", ARG_INT, -1, (void*)backend);
	
	tag = gtk_idle_add((GtkFunction)idle_socket, ctrls);
	global_tag = tag;
	arg_add_value(ctrls, "TAG", ARG_INT, sizeof(int), (void*)tag);
		
	update_status ("starting scan");
	set_widgets_sensitive (FALSE,TRUE);
	
	w = glade_xml_get_widget (mainwindow_xml,"stop_scan_button");
	gtk_widget_set_sensitive (w, TRUE);
	w = glade_xml_get_widget (mainwindow_xml,"stop_scan1");
	gtk_widget_set_sensitive (w, TRUE);	
	w = glade_xml_get_widget (mainwindow_xml,"stop_scanning_host_button");
	gtk_widget_set_sensitive (w, TRUE);
	
	gtk_list_store_clear (scan_status_store);
	
	scan_hosts = g_hash_table_new (g_str_hash,g_str_equal);
	scan_running = TRUE;
	
	attack_host (hostname, Prefs);
}

void on_stop_scan_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	int type = 0, n;
	char * msg;
	char buf[32768];
	GtkWidget *w;
	
	network_printf( "CLIENT <|> STOP_WHOLE_TEST <|> CLIENT\n");
	efree(&ArgSock);
	gtk_idle_remove(global_tag);	
	
	 while(type != MSG_BYE) {
		bzero(buf, sizeof(buf));
		network_gets_raw(buf, sizeof(buf) - 1);
		
		 if( buf[0] == '\0') {
			break;
		}

		if ((n = strlen (buf)) && buf [n-1] == '\n') 
			buf [n-1] = '\0';

		msg  = emalloc(strlen(buf)+1);
		type = parse_server_message(buf, global_backend, msg);
		efree(&msg);
	}
  
	F_nessusd_running = 0;
	update_status ("Full scan stopped");
	set_widgets_sensitive (TRUE,FALSE);

	w = glade_xml_get_widget (mainwindow_xml,"stop_scan_button");
	gtk_widget_set_sensitive (w, FALSE);
	w = glade_xml_get_widget (mainwindow_xml,"stop_scan1");
	gtk_widget_set_sensitive (w, FALSE);	
	w = glade_xml_get_widget (mainwindow_xml,"stop_scanning_host_button");
	gtk_widget_set_sensitive (w, FALSE);	

	hosts = NULL;
	scan_running = FALSE;
	
	show_results (global_backend, 0);		
}

void stop_scanning_host (GtkTreeModel *model,GtkTreePath *path, GtkTreeIter *iter, gpointer data) {

	if (gtk_list_store_iter_is_valid (GTK_LIST_STORE(model),iter)) {

		gchar *cur_hostname;
		
		gtk_tree_model_get (model,iter,
							0,&cur_hostname,
							-1);
		
		network_printf("CLIENT <|> STOP_ATTACK <|> %s <|> CLIENT\n", cur_hostname);
		gtk_list_store_set (GTK_LIST_STORE(model), iter,
							1, "Cancelled",  
							2, "Cancelled",
							-1);
		
		g_free (cur_hostname);
	} else {
		update_status ("No host was selected to stop a scan on");
	}	
}

void on_stop_scanning_host_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(scan_status_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	gtk_tree_selection_selected_foreach (selected,stop_scanning_host,NULL);	
}

void on_quit1_activate (GtkWidget *widget, gpointer user_data) {
	
	gtk_main_quit ();
}

void on_connect1_activate (GtkWidget *widget, gpointer user_data) {
	
	on_connect_button_clicked (widget,NULL);
}

void on_disconnect1_activate (GtkWidget *widget, gpointer user_data) {
	
	on_disconnect_button_clicked (widget,NULL);
}

void on_start_scan1_activate (GtkWidget *widget, gpointer user_data) {
	on_start_scan_button_clicked (widget,NULL);
}

void on_stop_scan1_activate (GtkWidget *widget, gpointer user_data) {
	
	on_stop_scan_button_clicked (widget,NULL);
}

void update_server_list_callback (gpointer data, gpointer user_data) {

	GdkPixbuf *connected_pixbuf;	
	GtkTreeIter iter;
	gchar *tmp;
	gchar *host;

	connected_pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/logout.png",NULL);
	
	tmp = g_strdup ((gchar*)data);
	host = strtok (tmp, ":");
	
	gtk_list_store_append (GTK_LIST_STORE(nessusd_store), &iter);
	gtk_list_store_set (nessusd_store, &iter,
						0, connected_pixbuf,
						1, host,
						-1);
}

void update_server_listview (void) {

	GSList *list = NULL;
	
	list = get_server_xml ();
	gtk_list_store_clear (GTK_LIST_STORE(nessusd_store));
	g_slist_foreach  (list, update_server_list_callback, NULL);	
}

void on_preferences1_activate (GtkWidget *widget, gpointer user_data) {
	
	show_preferences_dialog ();
	update_server_listview ();	
}

void on_copy1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_cut1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_paste1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_clear1_activate (GtkWidget *widget, gpointer user_data) {
}

void on_properties1_activate (GtkWidget *widget, gpointer user_data) {
	on_properties_button_clicked (widget, NULL);
}

void on_refresh_server_list_activate (GtkWidget *widget, gpointer user_data) {
	update_server_listview ();
}

void on_help_contents_activate (GtkWidget *widget, gpointer user_data) {
	
	GError *err = NULL;
	gboolean status;
		
	status = gnome_help_display ("sussen.xml",NULL,&err);

	if (!status) {
		g_print ("faliure: [%s]\n", err->message);
		g_free (err);
	}
}

void on_restore_session_button_clicked (GtkWidget *widget, gpointer user_data) {

	struct arglist *serv;
	gchar expansion[30];
	gchar *s;
	gchar *hostname;
	GtkWidget *w;

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	gchar *session_name;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(session_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	
	if (gtk_list_store_iter_is_valid (session_store,&iter)) {
		
		gtk_tree_model_get (GTK_TREE_MODEL(session_store),&iter,
							0,&session_name,
							-1);
	
		bzero (expansion, 30);
		serv = arg_get_value(Prefs, "SERVER_PREFS");
		if (!serv) {
			g_print ("!serv\n");
			serv = emalloc(sizeof(struct arglist));
			arg_add_value(Prefs, "SERVER_PREFS", ARG_ARGLIST, -1, serv);				
		}
		
		w = glade_xml_get_widget (mainwindow_xml,"dns_transfer_checkbutton");
		if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) )
			strcat (expansion, "dns;");
	   
		strcat(expansion, "ip;");    
   
		if (!strlen(expansion))
			strncpy(expansion, "none;", 5);

		expansion[strlen(expansion)-1]=0;	
	
		w = glade_xml_get_widget (mainwindow_xml,"targets_entry");
		s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
		hostname = target_translate(s); 
   
		s = emalloc(strlen(expansion)+1);
		strncpy(s, expansion, strlen(expansion));

		/* error check */
		if(GlobalSocket < 0) {
			GtkWidget *dialog;

			dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
		                      GTK_DIALOG_DESTROY_WITH_PARENT,
	       					  GTK_MESSAGE_ERROR,
							GTK_BUTTONS_CLOSE,
							_("You must be connected to a nessus server"));
	
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);		

			return;
		}
	 	
		struct arglist *ctrls = emalloc(sizeof(struct arglist));
		int tag;
		int backend = backend_init(NULL);
	
		global_backend = backend;
		arg_add_value(ctrls, "MONITOR_BACKEND", ARG_INT, -1, (void*)backend);
	
		tag = gtk_idle_add((GtkFunction)idle_socket, ctrls);
		global_tag = tag;
		arg_add_value(ctrls, "TAG", ARG_INT, sizeof(int), (void*)tag);
	
		restore_session = TRUE;
	
		restore_attack (session_name, Prefs);
		g_free (session_name);
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                          GTK_DIALOG_DESTROY_WITH_PARENT,
            		                      GTK_MESSAGE_ERROR,
                		                  GTK_BUTTONS_CLOSE,
                    		              _("You must select a session to restore"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
	}
}

void on_delete_session_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	GtkTreeSelection *selected;
	GtkTreeIter iter;
	gchar *key;
	gboolean result;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(session_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	if (gtk_list_store_iter_is_valid (session_store,&iter)) {

		gtk_tree_model_get (GTK_TREE_MODEL (session_store), &iter, 
							0, &key,
							-1);

		comm_delete_session (key);
		result = gtk_list_store_remove (session_store,&iter);
	
		g_free (key);
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                          GTK_DIALOG_DESTROY_WITH_PARENT,
            		                      GTK_MESSAGE_ERROR,
                		                  GTK_BUTTONS_CLOSE,
                    		              _("You must select a session to delete"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);				
	}
}

void on_session_button_clicked (GtkWidget *widget, gpointer user_data) {
	
	GtkWidget *dialog;
	GtkWidget *w;
		
    session_xml = glade_xml_new (PACKAGE_DATA_DIR"/session.glade", NULL, NULL);
	dialog = glade_xml_get_widget (session_xml, "session_dialog");
		
    glade_xml_signal_autoconnect(session_xml); 	
		
	/* destroy dialog on button press */
	w = glade_xml_get_widget (session_xml,"close_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked", 
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer)dialog);
	

	/*
	 * Configure session_listview
	 */
	session_tree = glade_xml_get_widget (session_xml,"session_listview");
	session_store = gtk_list_store_new (2,G_TYPE_STRING,G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW (session_tree), GTK_TREE_MODEL (session_store));
	{
		GtkTreeViewColumn *column;
		GtkCellRenderer *text_renderer;
		
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Session",text_renderer,
															"text",0,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 0);
		gtk_tree_view_append_column (GTK_TREE_VIEW (session_tree), column);	
			
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Targets",text_renderer,
																"text",1,NULL);
		gtk_tree_view_column_set_resizable (column, TRUE);
		gtk_tree_view_column_set_sort_column_id (column, 1);
		gtk_tree_view_append_column (GTK_TREE_VIEW (session_tree), column);
	}	
		
	
	/*
 	 * sessions
 	 */

	hargwalk *hw;
	harglst *sessions = Sessions;
	gchar *key;
	
	if(!sessions) {
		update_status ("No Sessions");
	}
	
	hw = harg_walk_init(sessions);
	while((key = (gchar*)harg_walk_next(hw))) {
	
		GtkTreeIter iter;
		
		gtk_list_store_append (session_store, &iter);
		gtk_list_store_set (session_store, &iter,
							0, key,  
							1, harg_get_string(sessions, key),
							-1);
	}	
}

void on_sessions1_activate (GtkWidget *widget, gpointer user_data) {
	on_session_button_clicked (widget,NULL);
}

void set_defaults (struct arglist *prefs) {

	struct arglist *serv;
	char *v;
	GtkWidget *widget;

	serv = arg_get_value(prefs, "SERVER_PREFS");
	v = arg_get_value(serv, "host_expansion");

	/* 'widgets on 'Scan Options' tab */

	widget = glade_xml_get_widget (xml,"max_aged_kb_entry");
	if (arg_get_value (prefs,"kb_max_age")) {
		gtk_entry_set_text ( GTK_ENTRY(widget), "864000" );
	} else {
		gtk_entry_set_text ( GTK_ENTRY(widget), "864000" );
	}

	widget = glade_xml_get_widget (xml,"reverse_ip_lookup_checkbutton");
	v = arg_get_value(serv, "reverse_lookup");
	if(arg_get_type(serv, "reverse_lookup")==ARG_INT) {
		char * s = emalloc(4);
		if(v)
			strncpy(s, "yes", 3);
		else 
			strncpy(s, "no", 2);
   
		arg_set_value(serv, "reverse_lookup", strlen(s), s);
		arg_set_type(serv, "reverse_lookup", ARG_STRING);
	}
	v = arg_get_value(serv, "reverse_lookup");
	if(v)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), !strcmp(v, "yes"));

	widget = glade_xml_get_widget (xml,"optimize_test_checkbutton");
	v = arg_get_value(serv, "optimize_test");
	if(arg_get_type(serv, "optimize_test")==ARG_INT) {
		char * s = emalloc(4);
		if(v)
			strncpy(s, "yes", 3);
		else 
			strncpy(s, "no", 2);
   
		arg_set_value(serv, "optimize_test", strlen(s), s);
		arg_set_type(serv, "optimize_test", ARG_STRING);
	}
	v = arg_get_value(serv, "optimize_test");
	if(v)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), !strcmp(v, "yes"));	

	widget = glade_xml_get_widget (xml,"safe_checks_checkbutton");
	v = arg_get_value(serv, "safe_checks");
	if(arg_get_type(serv, "safe_checks")==ARG_INT) {
		char * s = emalloc(4);

		if(v)
			strncpy(s, "yes", 3);
		else 
			strncpy(s, "no", 2);
   
		arg_set_value(serv, "safe_checks", strlen(s), s);
		arg_set_type(serv, "safe_checks", ARG_STRING);
	}
	v = arg_get_value(serv, "safe_checks");
	if(v)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), !strcmp(v, "yes"));
	else
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON(widget), 1);

	widget = glade_xml_get_widget (xml,"designate_by_mac_checkbutton");
	v = arg_get_value(serv, "use_mac_addr");
	if(arg_get_type(serv, "use_mac_addr")==ARG_INT) {
		char * s = emalloc(4);
		if(v)
			strncpy(s, "yes", 3);
		else 
			strncpy(s, "no", 2);
   
		arg_set_value(serv, "use_mac_addr", strlen(s), s);
		arg_set_type(serv, "use_mac_addr", ARG_STRING);
	}
  
	v = arg_get_value(serv, "use_mac_addr");
	if(v)
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), !strcmp(v, "yes"));
	else 
		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (widget), 0);	

	v = arg_get_value(serv, "port_range");
	if(v) {
		widget = glade_xml_get_widget (xml,"port_range_entry");
		gtk_entry_set_text (GTK_ENTRY(widget), v);
	}

	widget = glade_xml_get_widget (xml,"unscanned_ports_closed_checkbutton");
	v = arg_get_value(serv, "unscanned_closed");
	if(arg_get_type(serv, "unscanned_closed")==ARG_INT) {
		char * s;
		int v;
		v = (int)arg_get_value(serv, "unscanned_closed");

		if(v)
			s = strdup("yes");
		else
			s = strdup("no");
    
		arg_set_value(serv, "unscanned_closed", strlen(s), s);
		arg_set_type(serv, "unscanned_closed", ARG_STRING);
	}
	v = arg_get_value(serv, "unscanned_closed");
	if (v)
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), !strcmp(v, "yes"));
	else 
		gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(widget), 0);
  
	v = arg_get_value(serv, "max_hosts");
	if(v) {
		widget = glade_xml_get_widget (xml,"hosts_to_test_entry");
		gtk_entry_set_text (GTK_ENTRY(widget), v);
	}

	v = arg_get_value(serv, "max_checks");
	if(v) {
		widget = glade_xml_get_widget (xml,"checks_to_perform_entry");
		gtk_entry_set_text (GTK_ENTRY(widget), v);
	}
	
	v = arg_get_value(serv, "cgi_path");
	if(v) {
		widget = glade_xml_get_widget (xml,"cgibin_path_entry");
		gtk_entry_set_text (GTK_ENTRY(widget), v);
	}
}

void serv_callback (gpointer data, gpointer user_data) {

	gchar *host;
	
	gchar *cp;
	gchar *tmp;

	cp = g_strdup ((gchar*)data);
	tmp = g_malloc (255);	
		
	host = strtok (cp, ":");  
	if (strcmp (host,(gchar*)user_data) == 0) {
		nessusd_port = g_strdup(strtok (NULL, ":"));
		nessusd_user = g_strdup(strtok (NULL, ":"));
	}
	
	g_free (tmp);	
}

void on_connect_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	GdkPixbuf *login_pixbuf;
	gchar *hostname;
	GSList *list = NULL;
	
	selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(nessusd_tree));
	gtk_tree_selection_get_selected (selected,NULL,&iter);
	
	if (gtk_list_store_iter_is_valid (nessusd_store,&iter)) {
	
		gtk_tree_model_get (GTK_TREE_MODEL(nessusd_store),&iter,
							1, &hostname,
							-1);
		
		list = get_server_xml ();
		nessusd_host = g_strdup (hostname);
		g_slist_foreach (list,serv_callback,(gpointer)hostname);
		
		login_nessusd (nessusd_host, nessusd_port, nessusd_user);	
		
		login_pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/login.png",NULL);
		gtk_list_store_set (nessusd_store, &iter,
							0, login_pixbuf,  
							-1);
		
		g_free (hostname);
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        		                          GTK_DIALOG_DESTROY_WITH_PARENT,
            		                      GTK_MESSAGE_ERROR,
                		                  GTK_BUTTONS_CLOSE,
                    		              _("You must select a nessus server to connect to"));
 		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		update_status ("Nothing Selected");
	}
}

void on_disconnect_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkTreeSelection *selected;
	GtkTreeIter iter;
	GdkPixbuf *logout_pixbuf;
	
	if (!scan_running) {
		selected = gtk_tree_view_get_selection (GTK_TREE_VIEW(nessusd_tree));
		gtk_tree_selection_get_selected (selected,NULL,&iter);

		logout_pixbuf = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/logout.png",NULL);
		if (gtk_list_store_iter_is_valid (nessusd_store,&iter)) {

			logout_nessusd ();
			gtk_list_store_set (nessusd_store, &iter,
								0, logout_pixbuf,  
								-1);
			set_widgets_sensitive (FALSE,FALSE);
			GtkWidget *w = glade_xml_get_widget (mainwindow_xml, "connect_button");
			gtk_widget_set_sensitive (w,TRUE);
			w = glade_xml_get_widget (mainwindow_xml, "connect1");
			gtk_widget_set_sensitive (w,TRUE);	
		} else {
			GtkWidget *dialog;

			dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
	        			                      GTK_DIALOG_DESTROY_WITH_PARENT,
        	               					  GTK_MESSAGE_ERROR,
											   GTK_BUTTONS_CLOSE,
												_("You must select a nessus server to disconnect from"));
			gtk_dialog_run (GTK_DIALOG (dialog));
			gtk_widget_destroy (dialog);		
		}
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(main_window),
        			                      GTK_DIALOG_DESTROY_WITH_PARENT,
                       					  GTK_MESSAGE_WARNING,
										   GTK_BUTTONS_CLOSE,
											_("There is currently a scan running, stop that first"));
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
	}
}

void get_server_properties (void) {

	GtkWidget *w;
	gchar *s;
	struct arglist *prefs = arg_get_value(Prefs, "SERVER_PREFS");

	/*
	 * Scan Options tab
	 */
	
	w = glade_xml_get_widget (xml,"reverse_ip_lookup_checkbutton");
	s = emalloc(4);
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) 
		strncpy (s, "yes", 4);
	else 
		strncpy(s, "no", 4);
   
	if (arg_get_value(prefs, "reverse_lookup"))
		arg_set_value(prefs, "reverse_lookup", strlen(s), s);
	else
		arg_add_value(prefs, "reverse_lookup", ARG_STRING, strlen(s), s);

	w = glade_xml_get_widget (xml,"optimize_test_checkbutton");
	s = emalloc(4);
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) 
		strncpy (s, "yes", 4);
	else 
		strncpy(s, "no", 4);
   
	if(arg_get_value(prefs, "optimize_test"))
		arg_set_value(prefs, "optimize_test", strlen(s), s);
	else
		arg_add_value(prefs, "optimize_test", ARG_STRING, strlen(s), s);

	w = glade_xml_get_widget (xml,"safe_checks_checkbutton");
	s = emalloc(4);
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) 
		strncpy (s, "yes", 4);
	else 
		strncpy(s, "no", 4);
   
	if(arg_get_value(prefs, "safe_checks"))
		arg_set_value(prefs, "safe_checks", strlen(s), s);
	else
		arg_add_value(prefs, "safe_checks", ARG_STRING, strlen(s), s);
	
	w = glade_xml_get_widget (xml,"designate_by_mac_checkbutton");
	s = emalloc(4);
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) 
		;
	else 
		strncpy(s, "no", 4);
   
	if(arg_get_value(prefs, "use_mac_addr"))
		arg_set_value(prefs, "use_mac_addr", strlen(s), s);
	else
		arg_add_value(prefs, "use_mac_addr", ARG_STRING, strlen(s), s);

	w = glade_xml_get_widget (xml,"detached_scan_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) {
		s = strdup("yes");
		DetachedMode = 1;
	}
	else {
		s = strdup("no");
		DetachedMode = 0;
	}
		
	if(arg_get_value(prefs, "detached_scan"))
		arg_set_value(prefs, "detached_scan", strlen(s), s);
	else
		arg_add_value(prefs, "detached_scan", ARG_STRING, strlen(s), s);  
  
	w = glade_xml_get_widget (xml,"email_results_to_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));
	arg_set_value(prefs, "detached_scan_email_address", strlen(s), s);	

	w = glade_xml_get_widget (xml,"continuous_scan_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) ) 
		s = strdup("yes");
	else 
		s = strdup("no");
	
	if(arg_get_value(prefs, "continuous_scan"))
		arg_set_value(prefs, "continuous_scan", strlen(s), s);
	else
		arg_add_value(prefs, "continuous_scan", ARG_STRING, strlen(s), s);
	
	w = glade_xml_get_widget (xml,"delay_between_scans_entry");
	s = (gchar *)gtk_entry_get_text(GTK_ENTRY(w));

	if(arg_get_value(prefs, "delay_between_scan_loops"))
		arg_set_value(prefs, "delay_between_scan_loops", strlen(s), s);
	else
		arg_add_value(prefs, "delay_between_scan_loops", ARG_STRING, strlen(s), s); 

	w = glade_xml_get_widget (xml,"port_range_entry");
	s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(w)))+1);
	strncpy(s, gtk_entry_get_text(GTK_ENTRY(w)), 
	strlen(gtk_entry_get_text(GTK_ENTRY(w))));
 
	if(arg_get_value(prefs, "port_range"))
		arg_set_value(prefs, "port_range", strlen(s), s);
	else
		arg_add_value(prefs, "port_range", ARG_STRING, strlen(s), s);      

	w = glade_xml_get_widget (xml,"unscanned_ports_closed_checkbutton");
	s = emalloc(4);
	if( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (w)) )
		strncpy(s, "yes", 3);
	else 
		strncpy(s, "no", 4);
   
	if(arg_get_value(prefs, "unscanned_closed"))
		arg_set_value(prefs, "unscanned_closed", strlen(s), s);
	else
		arg_add_value(prefs, "unscanned_closed", ARG_STRING, strlen(s), s);
    
    w = glade_xml_get_widget (xml,"hosts_to_test_entry");	 
	s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(w)))+1);
  
	strncpy(s, gtk_entry_get_text(GTK_ENTRY(w)), 
	strlen(gtk_entry_get_text(GTK_ENTRY(w))));
  
	if(arg_get_value(prefs, "max_hosts"))
		arg_set_value(prefs, "max_hosts", strlen(s), s);
	else
		arg_add_value(prefs, "max_hosts", ARG_STRING, strlen(s), s); 
    
    w = glade_xml_get_widget (xml,"checks_to_perform_entry");
	s = emalloc(strlen(gtk_entry_get_text(GTK_ENTRY(w)))+1);
  
	strncpy(s, gtk_entry_get_text(GTK_ENTRY(w)), 
	strlen(gtk_entry_get_text(GTK_ENTRY(w))));
  
	if(arg_get_value(prefs, "max_checks"))
		arg_set_value(prefs, "max_checks", strlen(s), s);
	else
		arg_add_value(prefs, "max_checks", ARG_STRING, strlen(s), s);      
 
 	w = glade_xml_get_widget (xml,"cgibin_path_entry");
	s = estrdup((char*)gtk_entry_get_text(GTK_ENTRY(w)));
  
	if(arg_get_value(prefs, "cgi_path"))
		arg_set_value(prefs, "cgi_path", strlen(s), s);
	else
		arg_add_value(prefs, "cgi_path", ARG_STRING, strlen(s), s);   	
	
	/*
	 * Knowledge Base tab
	 */
	
	w = glade_xml_get_widget (xml,"enable_kb_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w) ) ) {
		arg_addset_value(prefs, "save_knowledge_base", "yes");
	} else {
		arg_addset_value(prefs, "save_knowledge_base", "no");
	}
	
	w = glade_xml_get_widget (xml,"kb_test_radio");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w) ) ) {
		arg_addset_value(prefs, "only_test_hosts_whose_kb_we_dont_have", "no");
		arg_addset_value(prefs, "only_test_hosts_whose_kb_we_have", "no");
	}
	
	w = glade_xml_get_widget (xml,"kb_test_past_hosts_radio");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w) ) ) {
		arg_addset_value(prefs, "only_test_hosts_whose_kb_we_dont_have", "no");
		arg_addset_value(prefs, "only_test_hosts_whose_kb_we_have", "yes");
	}	
	
	w = glade_xml_get_widget (xml,"kb_test_new_hosts_radio");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)) ) {
		arg_addset_value(prefs, "only_test_hosts_whose_kb_we_dont_have", "yes");
		arg_addset_value(prefs, "only_test_hosts_whose_kb_we_have", "no");
	}	

	w = glade_xml_get_widget (xml,"reuse_kb_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)) ) {
		arg_addset_value(prefs, "kb_restore", "yes");
	} else {
		arg_addset_value(prefs, "kb_restore", "no");
	}

	w = glade_xml_get_widget (xml,"no_scanners_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)) ) {
		arg_addset_value(prefs, "kb_dont_replay_scanners", "yes");
	} else {
		arg_addset_value(prefs, "kb_dont_replay_scanners", "no");
	}	

	w = glade_xml_get_widget (xml,"no_info_plugins_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)) ) {
		arg_addset_value(prefs, "kb_dont_replay_info_gathering", "yes");
	} else {
		arg_addset_value(prefs, "kb_dont_replay_info_gathering", "no");
	}	

	w = glade_xml_get_widget (xml,"no_attack_plugins_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)) ) {
		arg_addset_value(prefs, "kb_dont_replay_attacks", "yes");
	} else {
		arg_addset_value(prefs, "kb_dont_replay_attacks", "no");
	}	

	w = glade_xml_get_widget (xml,"no_dos_plugins_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)) ) {
		arg_addset_value(prefs, "kb_dont_replay_denials", "yes");
	} else {
		arg_addset_value(prefs, "kb_dont_replay_denials", "no");
	}	

	w = glade_xml_get_widget (xml,"show_differences_checkbutton");
	if ( gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON(w)) ) {
		arg_addset_value(prefs, "diff_scan", "yes");
	} else {
		arg_addset_value(prefs, "diff_scan", "no");
	}	

	w = glade_xml_get_widget (xml,"max_aged_kb_entry");
	arg_addset_value(prefs, "kb_max_age", (char *)gtk_entry_get_text(GTK_ENTRY(w)));
	
}

void on_properties_help_button_clicked (GtkWidget *widget, gpointer user_data) {

	GError *err = NULL;

	if (!gnome_help_display ("sussen.xml","sussen-server-properties",&err)) {
		g_print ("faliure: [%s]\n", err->message);
		g_free (err);
	}
}

void on_properties_button_clicked (GtkWidget *widget, gpointer user_data) {

	GtkWidget *properties_dialog;
	GtkWidget *w;
	
	/* 
	 * Create prefs dialog
	 */

	xml = glade_xml_new(PACKAGE_DATA_DIR"/server_properties.glade", NULL, NULL);
	properties_dialog = glade_xml_get_widget (xml,"sussen_server_properties_dialog");

    /* connect the signals in the interface */
    glade_xml_signal_autoconnect(xml); 	
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	w = glade_xml_get_widget (xml,"properties_ok_button");
	g_signal_connect (GTK_OBJECT (w),
					  "clicked",
					  G_CALLBACK (get_server_properties),
					  NULL);
		
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) properties_dialog);
	
	w = glade_xml_get_widget (xml,"properties_cancel_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) properties_dialog);
	

	set_defaults (Prefs);		
}
