/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-report-druid.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */
 
#include "sussen.h"
#include "sussen-util.h"
#include "sussen-report-druid.h"

/* nessus includes */

#include <includes.h>
#include "nessus.h"
#include "globals.h"
#include "backend.h"
#include "html_output.h"
#include "xml_output_ng.h"
#include "nbe_output.h"
#include "report_utils.h"


GtkWidget *druid;
GtkWidget *file_dialog;
gint user_druid_choice;
gint user_export_choice;
 
/*
 * FIXME: Seems to be a bug in libglade, the druid doesn't display the
 *        start and end pages.  Had to 'build source' from glade2 to make 
 *		  it work.  Odd that it works when building from source but not loading
 *		  from xml.
 */
 
GtkWidget* create_sussen_report_druid_dialog (void) {
	
	GtkWidget *report_druid_dialog;
	GtkWidget *dialog_vbox1;
	GtkWidget *report_druid;
	GtkWidget *report_start_page;
	GtkWidget *report_page1;
	GtkWidget *druid_vbox1;
	GtkWidget *vbox1;
	GtkWidget *label1;
	GtkWidget *vbox2;
	GtkWidget *vbox3;
	GtkWidget *view_report_radio;
	GSList *view_report_radio_group = NULL;
	GtkWidget *export_report_radio;
	GtkWidget *view_export_radio;
	GtkWidget *report_page2;
	GtkWidget *druid_vbox2;
	GtkWidget *vbox4;
	GtkWidget *label2;
	GtkWidget *vbox5;
	GtkWidget *hbox1;
	GtkWidget *label4;
	GtkWidget *report_file_entry;
	GtkWidget *hbuttonbox1;
	GtkWidget *report_file_browse_button;
	GtkWidget *alignment1;
	GtkWidget *hbox2;
	GtkWidget *image1;
	GtkWidget *label5;
	GtkWidget *vbox6;
	GtkWidget *frame1;
	GtkWidget *vbox7;
	GtkWidget *report_html_radio;
	GSList *report_html_radio_group = NULL;
	GtkWidget *report_xml_radio;
	GtkWidget *report_nbe_radio;
	GtkWidget *label6;
	GtkWidget *report_end_page;
	GtkWidget *dialog_action_area1;

	report_druid_dialog = gtk_dialog_new ();
	gtk_window_set_title (GTK_WINDOW (report_druid_dialog), _("Report Druid"));
	
	dialog_vbox1 = GTK_DIALOG (report_druid_dialog)->vbox;
	gtk_widget_show (dialog_vbox1);

	report_druid = gnome_druid_new ();
	gtk_widget_show (report_druid);
	gtk_box_pack_start (GTK_BOX (dialog_vbox1), report_druid, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (report_druid), 4);

	report_start_page = gnome_druid_page_edge_new (GNOME_EDGE_START);
	gtk_widget_show (report_start_page);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
							GNOME_DRUID_PAGE (report_start_page));
	gnome_druid_page_edge_set_title (GNOME_DRUID_PAGE_EDGE (report_start_page),
									_("Report Druid"));
	gnome_druid_page_edge_set_text (GNOME_DRUID_PAGE_EDGE (report_start_page),
									_("Welcome to the report druid.  This will help you generate and/or view a report using the data from the scan you just completed."));

	report_page1 = gnome_druid_page_standard_new ();
	gtk_widget_show_all (report_page1);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
	GNOME_DRUID_PAGE (report_page1));

	druid_vbox1 = GNOME_DRUID_PAGE_STANDARD (report_page1)->vbox;
	gtk_widget_show (druid_vbox1);
	gtk_container_set_border_width (GTK_CONTAINER (druid_vbox1), 16);

	vbox1 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox1);
	gtk_box_pack_start (GTK_BOX (druid_vbox1), vbox1, TRUE, TRUE, 0);

	label1 = gtk_label_new (_("What would you like to do with the scan data?"));
	gtk_widget_show (label1);
	gtk_box_pack_start (GTK_BOX (vbox1), label1, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label1), GTK_JUSTIFY_LEFT);

	vbox2 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox2);
	gtk_box_pack_start (GTK_BOX (vbox1), vbox2, TRUE, TRUE, 0);

	vbox3 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox3);
	gtk_box_pack_start (GTK_BOX (vbox2), vbox3, TRUE, TRUE, 0);

	view_report_radio = gtk_radio_button_new_with_mnemonic (NULL, _("View the report"));
	gtk_widget_show (view_report_radio);
	gtk_box_pack_start (GTK_BOX (vbox3), view_report_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (view_report_radio), view_report_radio_group);
	view_report_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (view_report_radio));

	export_report_radio = gtk_radio_button_new_with_mnemonic (NULL, _("Export the report"));
	gtk_widget_show (export_report_radio);
	gtk_box_pack_start (GTK_BOX (vbox3), export_report_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (export_report_radio), view_report_radio_group);
	view_report_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (export_report_radio));

	view_export_radio = gtk_radio_button_new_with_mnemonic (NULL, _("Export and view the report"));
	gtk_widget_show (view_export_radio);
	gtk_box_pack_start (GTK_BOX (vbox3), view_export_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (view_export_radio), view_report_radio_group);
	view_report_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (view_export_radio));
	
	report_page2 = gnome_druid_page_standard_new ();
	gtk_widget_show_all (report_page2);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
	GNOME_DRUID_PAGE (report_page2));

	druid_vbox2 = GNOME_DRUID_PAGE_STANDARD (report_page2)->vbox;
	gtk_widget_show (druid_vbox2);
	gtk_container_set_border_width (GTK_CONTAINER (druid_vbox2), 16);

	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox4);
	gtk_box_pack_start (GTK_BOX (druid_vbox2), vbox4, TRUE, TRUE, 0);

	label2 = gtk_label_new (_("Specify the filename and select the type of report you want generated"));
	gtk_widget_show (label2);
	gtk_box_pack_start (GTK_BOX (vbox4), label2, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label2), GTK_JUSTIFY_LEFT);

	vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox5);
	gtk_box_pack_start (GTK_BOX (vbox4), vbox5, TRUE, TRUE, 0);

	hbox1 = gtk_hbox_new (FALSE, 0);
	gtk_widget_show (hbox1);
	gtk_box_pack_start (GTK_BOX (vbox5), hbox1, TRUE, TRUE, 0);

	label4 = gtk_label_new (_("Filename:"));
	gtk_widget_show (label4);
	gtk_box_pack_start (GTK_BOX (hbox1), label4, FALSE, FALSE, 6);
	gtk_label_set_justify (GTK_LABEL (label4), GTK_JUSTIFY_LEFT);

	report_file_entry = gtk_entry_new ();
	gtk_widget_show (report_file_entry);
	gtk_box_pack_start (GTK_BOX (hbox1), report_file_entry, TRUE, TRUE, 0);

	hbuttonbox1 = gtk_hbutton_box_new ();
	gtk_widget_show (hbuttonbox1);
	gtk_box_pack_start (GTK_BOX (hbox1), hbuttonbox1, TRUE, TRUE, 6);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (hbuttonbox1), GTK_BUTTONBOX_START);

	report_file_browse_button = gtk_button_new ();
	gtk_widget_show (report_file_browse_button);
	gtk_container_add (GTK_CONTAINER (hbuttonbox1), report_file_browse_button);
	GTK_WIDGET_SET_FLAGS (report_file_browse_button, GTK_CAN_DEFAULT);

	alignment1 = gtk_alignment_new (0.5, 0.5, 0, 0);
	gtk_widget_show (alignment1);
	gtk_container_add (GTK_CONTAINER (report_file_browse_button), alignment1);
	
	hbox2 = gtk_hbox_new (FALSE, 2);
	gtk_widget_show (hbox2);
	gtk_container_add (GTK_CONTAINER (alignment1), hbox2);

	image1 = gtk_image_new_from_stock ("gtk-open", GTK_ICON_SIZE_BUTTON);
	gtk_widget_show (image1);
	gtk_box_pack_start (GTK_BOX (hbox2), image1, FALSE, FALSE, 0);

	label5 = gtk_label_new_with_mnemonic (_("Browse"));
	gtk_widget_show (label5);
	gtk_box_pack_start (GTK_BOX (hbox2), label5, FALSE, FALSE, 0);
	gtk_label_set_justify (GTK_LABEL (label5), GTK_JUSTIFY_LEFT);

	vbox6 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox6);
	gtk_box_pack_start (GTK_BOX (vbox5), vbox6, TRUE, TRUE, 0);

	frame1 = gtk_frame_new (NULL);
	gtk_widget_show (frame1);
	gtk_box_pack_start (GTK_BOX (vbox6), frame1, TRUE, TRUE, 0);

	vbox7 = gtk_vbox_new (FALSE, 0);
	gtk_widget_show (vbox7);
	gtk_container_add (GTK_CONTAINER (frame1), vbox7);

	report_html_radio = gtk_radio_button_new_with_mnemonic (NULL, _("HTML"));
	gtk_widget_show (report_html_radio);
	gtk_box_pack_start (GTK_BOX (vbox7), report_html_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (report_html_radio), report_html_radio_group);
	report_html_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (report_html_radio));

	report_xml_radio = gtk_radio_button_new_with_mnemonic (NULL, _("XML"));
	gtk_widget_show (report_xml_radio);
	gtk_box_pack_start (GTK_BOX (vbox7), report_xml_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (report_xml_radio), report_html_radio_group);
	report_html_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (report_xml_radio));

	report_nbe_radio = gtk_radio_button_new_with_mnemonic (NULL, _("Nessus NBE"));
	gtk_widget_show (report_nbe_radio);
	gtk_box_pack_start (GTK_BOX (vbox7), report_nbe_radio, FALSE, FALSE, 6);
	gtk_radio_button_set_group (GTK_RADIO_BUTTON (report_nbe_radio), report_html_radio_group);
	report_html_radio_group = gtk_radio_button_get_group (GTK_RADIO_BUTTON (report_nbe_radio));

	label6 = gtk_label_new (_("Report Format"));
	gtk_widget_show (label6);
	gtk_frame_set_label_widget (GTK_FRAME (frame1), label6);
	gtk_label_set_justify (GTK_LABEL (label6), GTK_JUSTIFY_LEFT);

	report_end_page = gnome_druid_page_edge_new (GNOME_EDGE_FINISH);
	gtk_widget_show (report_end_page);
	gnome_druid_append_page (GNOME_DRUID (report_druid),
	GNOME_DRUID_PAGE (report_end_page));

	dialog_action_area1 = GTK_DIALOG (report_druid_dialog)->action_area;
	gtk_widget_show (dialog_action_area1);
	gtk_button_box_set_layout (GTK_BUTTON_BOX (dialog_action_area1), GTK_BUTTONBOX_END);

	/* Store pointers to all widgets, for use by lookup_widget(). */
	GLADE_HOOKUP_OBJECT_NO_REF (report_druid_dialog, report_druid_dialog, "report_druid_dialog");
	GLADE_HOOKUP_OBJECT_NO_REF (report_druid_dialog, dialog_vbox1, "dialog_vbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_druid, "report_druid");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_start_page, "report_start_page");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_page1, "report_page1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, druid_vbox1, "druid_vbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox1, "vbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label1, "label1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox2, "vbox2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox3, "vbox3");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, view_report_radio, "view_report_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, export_report_radio, "export_report_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, view_export_radio, "view_export_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_page2, "report_page2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, druid_vbox2, "druid_vbox2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox4, "vbox4");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label2, "label2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox5, "vbox5");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, hbox1, "hbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label4, "label4");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_file_entry, "report_file_entry");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, hbuttonbox1, "hbuttonbox1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_file_browse_button, "report_file_browse_button");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, alignment1, "alignment1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, hbox2, "hbox2");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, image1, "image1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label5, "label5");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox6, "vbox6");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, frame1, "frame1");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, vbox7, "vbox7");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_html_radio, "report_html_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_xml_radio, "report_xml_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_nbe_radio, "report_nbe_radio");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, label6, "label6");
	GLADE_HOOKUP_OBJECT (report_druid_dialog, report_end_page, "report_end_page");
	GLADE_HOOKUP_OBJECT_NO_REF (report_druid_dialog, dialog_action_area1, "dialog_action_area1");

	return report_druid_dialog;	
}

void report_page2_next (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {

	struct arglist * hosts = NULL;
	GSList *list;
	GtkWidget *report_druid;
	GtkWidget *file_entry;
	GtkWidget *button;
	GtkWidget *next_page;
	const gchar *filename;
		
	report_druid = lookup_widget (druid,"report_druid");
	button = lookup_widget (druid, "report_html_radio");
	list = gtk_radio_button_get_group (GTK_RADIO_BUTTON(button));
	
	file_entry = lookup_widget (druid,"report_file_entry");
	filename = gtk_entry_get_text (GTK_ENTRY(file_entry));

	if (strlen(filename) > 0) {
		next_page = lookup_widget (druid,"report_end_page");
	
		if (list) {
			while (list) {
			
				GtkWidget *button = list->data;
			
			
				if(GTK_TOGGLE_BUTTON(button)->active) {
				
					user_export_choice = (gint)g_object_get_data(G_OBJECT(button), "fileformat");
					
					switch (user_export_choice) {
					
						case REPORT_DRUID_FILE_HTML:
							hosts = backend_convert(global_backend);
							arglist_to_html(hosts, filename);
							break;

						case REPORT_DRUID_FILE_XML:
							backend_to_xml_ng (global_backend, filename);
							break;
					
						case REPORT_DRUID_FILE_NBE:	
							backend_to_nbe (global_backend, filename);
							break;
					
						default:
							break;
					}
				}
				list = list->next;
			}
		}
		
		gnome_druid_page_edge_set_text (GNOME_DRUID_PAGE_EDGE (next_page),
									_("Your report has been exported.  Click 'Apply' to finish the druid."));
	} else {
		GtkWidget *dialog;

		dialog = gtk_message_dialog_new (GTK_WINDOW(druid),
	                      GTK_DIALOG_DESTROY_WITH_PARENT,
       					  GTK_MESSAGE_ERROR,
							GTK_BUTTONS_CLOSE,
							_("You must specify a filename"));
	
		gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);		
		
		next_page = lookup_widget (druid,"report_page2");
	}
	
	gnome_druid_set_page (GNOME_DRUID(report_druid),GNOME_DRUID_PAGE (next_page));	
}

void report_page1_next (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {

	GSList *list;
	GtkWidget *report_druid;
	GtkWidget *button;
	GtkWidget *next_page = lookup_widget(druid, "report_page2");
		
	report_druid = lookup_widget (druid,"report_druid");
	button = lookup_widget (druid, "view_report_radio");
	list = gtk_radio_button_get_group (GTK_RADIO_BUTTON(button));
	
	if (list) {
		while (list) {
			
			GtkWidget *button = list->data;
			
			
			if(GTK_TOGGLE_BUTTON(button)->active) {
				
				user_druid_choice = (gint)g_object_get_data(G_OBJECT(button), "response");
				
				switch (user_druid_choice) {
				
					case REPORT_DRUID_VIEW:
						next_page = lookup_widget (druid,"report_end_page");
						gnome_druid_page_edge_set_text (GNOME_DRUID_PAGE_EDGE (next_page),
														_("You have chosen to just view the report, click apply to launch the report viewer"));
						break;

					case REPORT_DRUID_EXPORT:
					case REPORT_DRUID_VIEW_EXPORT:	
						next_page = lookup_widget (druid,"report_page2");
						break;
				}
			}
			list = list->next;
		}
	}
	
	gnome_druid_set_page (GNOME_DRUID(report_druid),GNOME_DRUID_PAGE (next_page));	
}

void end_page_back (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {
	
	GtkWidget *report_druid;
	GtkWidget *back_page = NULL;
	
	switch (user_druid_choice) {
			
		case REPORT_DRUID_VIEW:
			back_page = lookup_widget (druid,"report_page1");
			break;

		case REPORT_DRUID_EXPORT:
		case REPORT_DRUID_VIEW_EXPORT:	
			back_page = lookup_widget (druid,"report_page2");
			break;
	}
	
	report_druid = lookup_widget (druid,"report_druid");
	gnome_druid_set_page (GNOME_DRUID(report_druid),GNOME_DRUID_PAGE (back_page));	
}

void end_page (GnomeDruidPage *druidpage, GtkWidget *widget, gpointer user_data) {

	gtk_widget_destroy (druid);
	gtk_main_quit ();
}

void set_filename_entry (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;
	const gchar *filename;
	
	filename = gtk_file_selection_get_filename (GTK_FILE_SELECTION(file_dialog));
	w = lookup_widget (druid,"report_file_entry");
	gtk_entry_set_text (GTK_ENTRY(w),filename);
}


void file_browse (GtkWidget *widget, gpointer user_data) {
	
	/* 
	 * Create file selector
	 */
	file_dialog = gtk_file_selection_new ("Select File to Save");
	g_signal_connect (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
					  "clicked",
					  G_CALLBACK (set_filename_entry),
					  NULL);
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->ok_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	g_signal_connect_swapped (GTK_OBJECT (GTK_FILE_SELECTION (file_dialog)->cancel_button),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) file_dialog);
	
	/*
	 * Show dialog
	 */
	 
	gtk_widget_show (file_dialog);
}

void on_cancel_clicked (GtkWidget *widget, gpointer user_data) {
	
	user_druid_choice = REPORT_DRUID_CANCEL;
	gtk_widget_destroy (druid);
	gtk_main_quit ();
}

gint show_report_druid (void) {
	
	GtkWidget *w;
	
	druid = create_sussen_report_druid_dialog ();
	gtk_widget_show (druid);

	/* configure radio buttons 
	 * 	1 = view report only
	 *  2 = export report only
	 *	3 = view & export report
	 *  4 = user canceled from druid
	 */
	
	w = lookup_widget (druid, "view_report_radio");
	g_object_set_data(G_OBJECT(w), "response", (void*)REPORT_DRUID_VIEW);
	
	w = lookup_widget (druid, "export_report_radio");
	g_object_set_data(G_OBJECT(w), "response", (void*)REPORT_DRUID_EXPORT);
	
	w = lookup_widget (druid, "view_export_radio");
	g_object_set_data(G_OBJECT(w), "response", (void*)REPORT_DRUID_VIEW_EXPORT);


	/* configure radio buttons 
	 * 	1 = HTML
	 *  2 = XML
	 *	3 = NBE
	 */
	
	w = lookup_widget (druid, "report_html_radio");
	g_object_set_data(G_OBJECT(w), "fileformat", (void*)REPORT_DRUID_FILE_HTML);
	
	w = lookup_widget (druid, "report_xml_radio");
	g_object_set_data(G_OBJECT(w), "fileformat", (void*)REPORT_DRUID_FILE_XML);
	
	w = lookup_widget (druid, "report_nbe_radio");
	g_object_set_data(G_OBJECT(w), "fileformat", (void*)REPORT_DRUID_FILE_NBE);

	w = lookup_widget (druid, "report_druid");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "cancel",
							  G_CALLBACK (on_cancel_clicked),
							  NULL);	

	
	w = lookup_widget (druid, "report_page1");
	g_signal_connect (GTK_OBJECT (w),
					  "next",
					  G_CALLBACK (report_page1_next),
					  NULL);	

	w = lookup_widget (druid, "report_page2");
	g_signal_connect (GTK_OBJECT (w),
					  "next",
					  G_CALLBACK (report_page2_next),
					  NULL);

	w = lookup_widget (druid, "report_end_page");
	g_signal_connect (GTK_OBJECT (w),
					  "back",
					  G_CALLBACK (end_page_back),
					  NULL);

	w = lookup_widget (druid, "report_end_page");
	g_signal_connect (GTK_OBJECT (w),
					  "finish",
					  G_CALLBACK (end_page),
					  NULL);

	w = lookup_widget (druid, "report_file_browse_button");
	g_signal_connect (GTK_OBJECT (w),
					  "clicked",
					  G_CALLBACK (file_browse),
					  NULL);	


	gtk_main ();
	
	return user_druid_choice;
}
