/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - report.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-report.h"

/* nessus includes */

#include <includes.h>
#include "nessus.h"
#include "globals.h"
#include "comm.h"
#include "parser.h"
#include "backend.h"
#include "data_mining.h"
#include "read_target_file.h"

GtkWidget *file_dialog;

#define n(x) (x ? x:"")

GtkWidget *report_tree_view;
GtkTreeStore *report_tree_store;

GtkWidget *report_data_view;
GtkListStore *report_data_store;

GtkCellRenderer *text_renderer;

GtkTreeIter report_parent_iter;
GtkTreeIter report_network_iter;
GtkTreeIter report_host_iter;
GtkTreeIter report_port_iter;
GtkTreeIter report_severity_iter;
GtkTreeIter report_data_iter;


static int 
cmp_hosts(a, b)
 char * a, * b;
{
 struct in_addr ia, ib;
 
 if(!a && !b)
 	return 0;
 
 if(!a) 
  return 1;
 if(!b)
   return -1;
  
 
 if(inet_aton(a, &ia) == 0)
  return strcmp(a, b);
 
 if(inet_aton(b, &ib) == 0)
 {
  return strcmp(a, b);
 }
 
 return -(ntohl(ia.s_addr) - ntohl(ib.s_addr));
}

static int
cmp_vulns(a, b)
 char * a, * b;
{
 int level_a, level_b;
 

 if(!a) 
  return -1;
 else if(!b)
  return 1;
  
 if(strstr(a, "Hole"))level_a = 3;
 else if(strstr(a, "Warning"))level_a = 2;
 else if(strstr(a, "Note"))level_a = 1;
 else level_a = 0;
 
 
 if(strstr(b, "Hole"))level_b = 3;
 else if(strstr(b, "Warning"))level_b = 2;
 else if(strstr(b, "Note"))level_b = 1;
 else level_b = 0;
 
 return level_a - level_b; 
}

/*
 * Converts a multiple subset (with field AND severity) to 
 * a sorted, uniq'ed one.
 *
 * The function name is set to 'x' just to simplify its use 
 * throughout the code
 */

static struct subset *x (struct subset *subset) {
	cmp_func_t cmps[] = {cmp_hosts, cmp_vulns};
	return subset_uniq(subset_sort(subset, 0, 1, cmps), 0);
}

void on_report_clicked (GtkTreeSelection *selection, gpointer user_data) {
	
	GtkTreeStore *local_tree_store;
	GtkListStore *local_list_store;
	GtkCellRenderer *local_text_renderer;
	
	GtkTreeIter iter;
	
	GtkTreeIter subnet_iter;
	GtkTreeIter host_iter;
	GtkTreeIter port_iter;
	GtkTreeIter sev_iter;
	
	gchar *iter_string;
	
	gchar *cur_subnet;
	gchar *cur_host;
	gchar *cur_port;
	gchar *cur_sev;
	
	local_tree_store = report_tree_store;
	local_list_store = report_data_store;
	local_text_renderer = text_renderer;
	
	gtk_tree_selection_get_selected (selection,NULL,&sev_iter);	
	if (gtk_tree_store_iter_is_valid (report_tree_store,&sev_iter)) {
		
		if (gtk_tree_store_iter_depth (report_tree_store, &sev_iter) == 4) {
	
			gtk_tree_model_get (GTK_TREE_MODEL(report_tree_store),&sev_iter,
								0, &cur_sev,
								-1);
					
			iter_string = gtk_tree_model_get_string_from_iter (GTK_TREE_MODEL(report_tree_store),
												&sev_iter);

			/* iter_string == root:subnet:host:port:severity 
			 *		eg: 0:0:0:0:0 
		  	 */
			
			gchar *cp;
			gchar *tmp;
			
			gchar *root;
			gchar *subnet;
			gchar *host;
			gchar *port;
			gchar *sev;
		
			tmp = g_malloc (20);
			
			cp = g_strdup (iter_string);
			root = strtok (cp, ":");  
			subnet = strtok (NULL, ":");
			host = strtok (NULL, ":");
			port = strtok (NULL, ":");
			sev = strtok (NULL, ":");

			
			sprintf (tmp,"%s:%s", root, subnet);		
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL(local_tree_store),
												&subnet_iter,g_strdup(tmp));
			gtk_tree_model_get (GTK_TREE_MODEL(local_tree_store),&subnet_iter,
								0, &cur_subnet,
								-1);
	
			sprintf (tmp,"%s:%s:%s", root, subnet, host);			
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL(local_tree_store),&host_iter,g_strdup(tmp));	
			gtk_tree_model_get (GTK_TREE_MODEL(local_tree_store),&host_iter,
								0, &cur_host,
								-1);
									
			sprintf (tmp,"%s:%s:%s:%s", root, subnet, host, port);
			gtk_tree_model_get_iter_from_string (GTK_TREE_MODEL(local_tree_store),&port_iter,g_strdup(tmp));	
			gtk_tree_model_get (GTK_TREE_MODEL(local_tree_store),&port_iter,
								0, &cur_port,
								-1);

			struct subset * subset = x(query_backend(global_backend, "SELECT %s,severity FROM results WHERE %s = '%s' AND %s = '%s' AND %s = '%s' AND %s = '%s'", 
								REPORT_SORT_KEY, 
								n(SUBNET_SORT_KEY), n(cur_subnet),
								n(HOST_SORT_KEY), n(cur_host),
								n(PORT_SORT_KEY), n(cur_port),
								n(SEVERITY_SORT_KEY), n(cur_sev)));	
			struct subset *walk = subset;
	
			gtk_list_store_clear (local_list_store);
	
			while(walk) {
	
				GdkPixbuf *icon;

				if (!strcmp(cur_sev, "Security Note")) {
					icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/info.png",NULL);			
				} else if (!strcmp(cur_sev, "Security Warning")) {
					icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/warning.png",NULL);
			    } else if (!strcmp(cur_sev, "Security Hole")) {
					icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/error.png",NULL);
				} else {
                    icon = gdk_pixbuf_new_from_file (PACKAGE_PIXMAPS_DIR"/nothing.xpm",NULL);
                }
				
				gtk_list_store_append (GTK_LIST_STORE(local_list_store), &iter);
				gtk_list_store_set (local_list_store, &iter,
									0, icon,
									1, g_strdup(subset_value(walk)),
									-1);	
		
				walk = subset_next(walk);
			}
	
			subset_free(subset);
			
			g_free (tmp);
			g_free (iter_string);
			g_free (cur_subnet);
			g_free (cur_host);
			g_free (cur_port);
			g_free (cur_sev);			
		}
	}	
}

gint show_results (gint be, gint interrupted) {

	GtkWidget *dialog;
	GtkWidget *w;
	
	if ( backend_empty (be) <= 0) {
		g_print ("empty report or detached scan running\n");
		return -1;
	}
	
	report_xml = glade_xml_new (PACKAGE_DATA_DIR"/report_viewer.glade", NULL,NULL);
	dialog = glade_xml_get_widget (report_xml, "report_viewer_dialog");
	
	w = glade_xml_get_widget (report_xml,"close_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) dialog);



	/* Setup report_tree widget */
	report_tree_view = glade_xml_get_widget (report_xml,"report_treeview");
	report_tree_store = gtk_tree_store_new (1, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW(report_tree_view), GTK_TREE_MODEL (report_tree_store));
	{
		GtkCellRenderer *text_renderer;
		GtkTreeViewColumn *column;
	
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Report",text_renderer,
															"text", 0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_tree_view), column);		
	}

	gtk_tree_store_append (report_tree_store, &report_parent_iter, NULL);
	gtk_tree_store_set (report_tree_store, &report_parent_iter,0, "Report",-1);	
	
	struct subset *network_subset = x(query_backend(be, "SELECT %s,severity FROM results", 
										SUBNET_SORT_KEY ));
	struct subset *network_walk;

	network_walk = network_subset;
	while(network_walk) {	

		gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_network_iter, 
								&report_parent_iter);	
		gtk_tree_store_set (report_tree_store, &report_network_iter,
							0, g_strdup(subset_value(network_walk)),
							-1);
		
		struct subset *host_subset = x(query_backend(global_backend, "SELECT %s,severity FROM results WHERE %s = '%s'",
										HOST_SORT_KEY, n(SUBNET_SORT_KEY), n(subset_value(network_walk))));
		struct subset *host_walk = host_subset;
		
		while (host_walk) {

			gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_host_iter, 
									&report_network_iter);	
			gtk_tree_store_set (report_tree_store, &report_host_iter,
								0, g_strdup(subset_value(host_walk)),
								-1);
			
			struct subset *port_subset = x(query_backend(global_backend, "SELECT %s,severity FROM results WHERE %s = '%s'  AND %s = '%s'",
											PORT_SORT_KEY,
											n(SUBNET_SORT_KEY),
											n(subset_value(network_walk)),
											n(HOST_SORT_KEY),
											n(subset_value(host_walk))));
			struct subset *port_walk = port_subset;
				
			while (port_walk) {

				struct subset *severity_subset = x(query_backend(global_backend, "SELECT %s,severity FROM results WHERE %s = '%s' AND %s = '%s' AND %s = '%s'",
													SEVERITY_SORT_KEY,
													n(SUBNET_SORT_KEY),
													n(subset_value(network_walk)),
													n(HOST_SORT_KEY),
													n(subset_value(host_walk)),
													n(PORT_SORT_KEY),
													n(subset_value(port_walk))));
				struct subset *severity_walk = severity_subset;
					
				if (severity_walk) {
				
					gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_port_iter, 
											&report_host_iter);	
					gtk_tree_store_set (report_tree_store, &report_port_iter,
										0, g_strdup(subset_value(port_walk)),
										-1);

					while (severity_walk) {
				
						gtk_tree_store_append (GTK_TREE_STORE(report_tree_store), &report_severity_iter, 
												&report_port_iter);	
						gtk_tree_store_set (report_tree_store, &report_severity_iter,
											0, g_strdup(subset_value(severity_walk)),
											-1);
				
						severity_walk = subset_next (severity_walk);
					}
				}
				
				subset_free (severity_subset);
				port_walk = subset_next (port_walk);
			}
			 
			subset_free (port_subset);
			host_walk = subset_next (host_walk);
		}
		
		subset_free (host_subset);
		network_walk = subset_next(network_walk);
	}
	
	subset_free (network_subset);

	GtkTreeSelection *report_selection = gtk_tree_view_get_selection (GTK_TREE_VIEW(report_tree_view));		
	gtk_tree_selection_set_mode (report_selection, GTK_SELECTION_SINGLE);
	g_signal_connect (GTK_TREE_VIEW(report_selection), "changed",  (void *)on_report_clicked, NULL);	
	
	/* Setup scan_data tree widget */
	report_data_view = glade_xml_get_widget (report_xml,"scan_data_listview");
	report_data_store = gtk_list_store_new (2, GDK_TYPE_PIXBUF, G_TYPE_STRING);
	gtk_tree_view_set_model (GTK_TREE_VIEW(report_data_view), GTK_TREE_MODEL (report_data_store));
	{
	    GtkCellRenderer *pixbuf_renderer;
		GtkTreeViewColumn *column;
		
		pixbuf_renderer = gtk_cell_renderer_pixbuf_new ();		
		column = gtk_tree_view_column_new_with_attributes ("Severity",pixbuf_renderer,
															"pixbuf",0,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_data_view), column);		
	
		text_renderer = gtk_cell_renderer_text_new ();
		column = gtk_tree_view_column_new_with_attributes ("Information",text_renderer,
															"text", 1,NULL);
		gtk_tree_view_append_column (GTK_TREE_VIEW (report_data_view), column);		
	}	
	
		
	return 0;
}
